/************************************************************************
**
** @(#)chkfpair.cpp		08/05/94		Chris Ahlstrom
**
** Edit with tab-stops at 4.
**
** ------------------------
** 73340.26@compuserve.com
** ------------------------
**
**		Manages a CheckListFloatPair dialog box.  This is the CheckListBase
** dialog class with support for converting between strings and floating
** point numbers.  See chklist.cpp/h and chkflt.cpp/h.
**
**		The CheckListFloatPair is similar to CheckListFloat, but handles
** two numbers (separated by a space) in one input line.  The pointer
** is to a FloatPair type, instead of just a float.
**
*************************************************************************/

#define CHKFPAIR_cpp

#include <math.h>				// atof()

#include "chkfpair.h"			// external declarations


/************************************************************************
** CheckListFloatPair()
**
**		The constructor is somewhat similar to that in chkflt.cpp.
**
*************************************************************************/

CheckListFloatPair::CheckListFloatPair
(
	TDeskTop *desk,				// TurboVision desktop
	int checkoptions,			// word containing desired check-list options
	const char *maintitle,		// string to use for main title
	const char *itemtitle,		// string to use for item title
	int *status,				// array word encoding status of each item
	FloatPair *items,			// pointer to array of items
	int maxitem,				// count of number of items
	int initpage,				// current page in list [default = 1]
	char *optionlabel			// one option label for now
) :
	CheckListBase
	(
		desk, checkoptions, maintitle, itemtitle,
		(CheckListBits *) status, (void *) items,
		maxitem, initpage, optionlabel
	)
{
}


CheckListFloatPair::~CheckListFloatPair ()
{
}


/************************************************************************
** setItemString()
**
**		Gets the item string *from* the caller.
**
**		It gets the item'th floating point pair in an array of them,
** and converts them to a string.
**
*************************************************************************/

#define CHK_FPAIR_FORMAT	"%9.5g %9.5g"				// display format

void
CheckListFloatPair::setItemString
(
	int item
)
{
	FloatPair *fparray = (FloatPair *) callerItems;	// pointer to 1st
	int status = callerStatus[firstItem+item];		// caller has set status

	if (status & CHKLIST_ACTIVE)
	{
		(void) sprintf
		(
			statusArea.itemString[item],			// destination string
			CHK_FPAIR_FORMAT,						// display format
			(fparray+firstItem+item)->a,			// the numbers
			(fparray+firstItem+item)->b				// the numbers
		);
	}
	else
		(void) strcpy(statusArea.itemString[item], "<inactive>");
}


/************************************************************************
** getItemString()
**
**		Gets the item string and reformats it to put it back *to* the
** caller.  This routine also puts back the status field, too.
**
*************************************************************************/

static inline int
isnumeric
(
	char digi
)
{
	return
	(
		(digi >= '0' && digi <= '9') ||
		(digi == '.') || (digi == '-') || (digi == '+') ||
		(digi == 'e') || (digi == 'E')
	);
}

void
CheckListFloatPair::getItemString
(
	int item
)
{
	FloatPair *fparray = (FloatPair *) callerItems;	// pointer to 1st pair
	char *text = &statusArea.itemString[item][0];	// pointer to item
	int countdown = CHK_ITEM_CHARS;					// maximum chars in item

	while (*text == ' ' && *text != '\0')			// skip initial white space
	{
		if (--countdown == 0)
			break;
		else
			text++;
	}

	(fparray+firstItem+item)->a =
	(fparray+firstItem+item)->b = -1.0f;

	if ((countdown == 0) || !isnumeric(*text))		// don't have good number
	{
		callerStatus[firstItem+item] = CHKLIST_UNUSED;
	}
	else											// have good number-string
	{
		(fparray+firstItem+item)->a = (float) atof(text);

		while (isnumeric(*text) && *text != '\0')	// skip first number
		{
			if (--countdown == 0)
				break;
			else
				text++;
		}
		if (countdown != 0)							// possible good 2nd number
			(fparray+firstItem+item)->b = (float) atof(text);

		callerStatus[firstItem+item] = statusArea.itemStatus[item];
	}
}

