/************************************************************************
**
** @(#)chklist.h		08/19/94		Chris Ahlstrom
**
**		Manages the CheckListDialog for the DOS version.  Similar in
** some ways to the x_calpk.cpp/h/men modules.  See them for further
** commentary.
**
*************************************************************************/

#ifndef CHKLIST_h									// { CHKLIST_h
#define CHKLIST_h

#include "chklbits.h"		// CheckListFunction and CheckListBits types
#include "dlg_box.h"		// DialogBox class of FACE_TV 


/************************************************************************
** CHKLIST_ITEMS_INDEX
**
**		Simply indicates that the radio-button box is the 1st (index
** is 0) field in the Calibration/Pick Device dialog box.  We need to
** know where it is, so we can load the main item-title location into
** that field.
**
**
** CHKLIST_OPTIONS_INDEX
**
**		The index of the "options" radio-button, so we can fill in some
** optional bits' labels.
**
*************************************************************************/

#define CHKLIST_ITEMS_INDEX		 0		// for access to items labels
#define CHKLIST_OPTIONS_INDEX	17		// for access to options labels

#define OPTION_BLANK "Option ~1~          "		// for "clearing" it


/************************************************************************
** Various sizes
*************************************************************************/

#define ITEM_DIGIT_LENGTH		 2		// number of digits for item numbers
#define ITEM_DIGIT_FORMAT		"%2d"	// format to use for writing them
#define PAGE_DIGIT_LENGTH		 2		// number of digits for page numbers
#define PAGE_DIGIT_FORMAT		"%2d"	// format to use for writing them
#define CHK_PAGE_SIZE			16		// number of items per page
#define CHK_ITEM_CHARS			20		// number of chars in item strings
#define CHK_TITLE_CHARS			32		// chars in main and item-title


/************************************************************************
** CheckListDTA
**
**		This structure is the format of the data-transfer area to be
** used in creating the checklist selection dialog box.
**
*************************************************************************/

typedef char ItemLabel[CHK_PAGE_SIZE][CHK_ITEM_CHARS+1];

typedef struct
{
	unsigned itemSelection;			// bits on mean "display the X mark"
	ItemLabel itemString;			// a 16x20 array for item labels
	unsigned checkOptions;			// options for functionality
	int currentPage;				// current 16-slot page
	int maximumPage;				// max page, dev. list

} CheckListDTA;


/************************************************************************
** CurrentPageData
**
**		This structure holds the data that is to be displayed in
** the dialog box.  It is a window onto an often larger set of
** data.
**
*************************************************************************/

typedef struct
{
	char mainTitle[CHK_TITLE_CHARS+1];			// main title of dialog box
	char itemTitle[CHK_TITLE_CHARS+1];			// title covering item section
	ItemLabel itemString;						// 16x20 array for item labels
	CheckListBits itemStatus[CHK_PAGE_SIZE];	// overall status of slot
	unsigned checkOptions;						// options for functionality
	int currentPage;							// current 16-slot page
	int maximumPage;							// max page, dev. list

} CurrentPageData; 


/************************************************************************
** CheckListBase
**
**		Since we need to get down and dirty and modify the dialog box
** on the fly (run-time), rather than use the FACE_WIN library tools,
** we build the dialog the old-fashioned OWL 1.0 way.
**
**		To support changes to the dialog controls, we must override
** SetupWindow(). In addition, we keep track of the device list
** and the state of activity (enabled/disabled) of the radio-buttons.
**
*************************************************************************/

class CheckListBase
{

public:

	CheckListBase
	(
		TDeskTop *desk,				// TurboVision desktop
		int checkoptions,			// word: desired check-list options
		const char *maintitle,		// string to use for main title
		const char *itemtitle,		// string to use for item title
		CheckListBits *status,		// array word encoding status of each item
		void *items,				// pointer to array of items
		int maxitem,				// count of number of items
		int initpage = 1,			// current page in list [default = 1]
		char *optionlabel = 0		// one option label for now
	);
	virtual ~CheckListBase();
	virtual int run ();
	int getCheckOptions ()
	{
		return (int) dialogData.checkOptions;
	};

protected:							// internal support functions, constant

	int setCurrentPage ();
	void getCurrentPage ();
	void setNewPage ();
	void copyItemLabels
	(
		ItemLabel &source,
		ItemLabel &dests
	);

protected:							// internal support functions, virtual

	virtual int itemIsChecked
	(
		int item
	);
	virtual int itemIsActive
	(
		int item
	);
	virtual void setItemString
	(
		int item
	);
	virtual void getItemString
	(
		int item
	);
	virtual int runDialog ();

protected:

	TDeskTop *deskTop;				// TurboVision desktop
	CheckListBits *callerStatus;	// list of status words caller uses
	void *callerItems;				// list of items the caller uses
	int currentOptions;				// initial options for check-list
	int initialPage;				// initial page in list, re 1
	int currentPage;				// current page in list, re 1
	int maximumPage;				// maximum page in list, re 1
	int itemCount;					// number of items actually in use
	int firstItem;					// index of first item on current page
	int bitCount;					// number of useable items on current page
	char *optionLabel;				// extra label for option button

public:

	CheckListDTA dialogData;		// immediate data for dialog
	CurrentPageData statusArea;		// tells all about dialog construction

};


#endif												// } CHKLIST_h
