/************************************************************************
**
** @(#)dlg_box.h		10/06/94		Chris Ahlstrom
**
**	C++
**
**		Declares the DialogBox class. See dlg_box.cpp for full details.
**
** NEW FEATURE:
**
**		The caller can now change the data in the dialog-related
** structure that contained the data for the dialog box, and call
** the member function updateFields() to cause all the fields to be
** redrawn in one command.
**
**		This feature is especially useful with another new feature:
** if doDialog() is called with a non-zero parameter, then the dialog
** is create as a "modeless" dialog, so that it can stay onscreen for
** later removal [by the undoDialog() function].  So, by having the
** caller alter the data as the program runs, and by calling
** updateFields(), the caller can implement a very complex status box.
**
*************************************************************************/

#if !defined(DLG_BOX_h)									// { DLG_BOX_h
#define DLG_BOX_h


#include "tv_box.h"		// Turbo Vision class support
#include "tv_menu.h"	// C++ version
#include "tinpmous.h"	// "Variations on TInputLine in C++", Op. 3.03


/************************************************************************
** GeneralDialogData
** TVFieldPtr
**
**		GeneralDialogData makes it easy to pass the address of data of
** any type to the DialogBox routines.
**
*************************************************************************/

typedef void *GeneralDialogData;


/************************************************************************
** DialogBox
*************************************************************************/

class DialogBox
{

public:

	DialogBox							// full-blown constructor
	(
		MainBox *dialogwindow,			// nested structure (tv_menu.h)
		GeneralDialogData dialog,		// points general dialog structure
		int size,						// number of bytes in data
		DeskTop *desk,					// pointer to main display
		ExtendedDescriptor *d = NULL,	// special extensions
		MappedFieldDescriptor *f = NULL	// whole new idea
	);
	DialogBox							// constructor without extended maps
	(
		MainBox *dialogwindow,			// nested structure (tv_menu.h)
		GeneralDialogData dialog,		// points to general dialog structure
		int size,						// number of bytes in data
		DeskTop *desk,
		MappedFieldDescriptor *fMap
	);
	virtual ~DialogBox();

public:									// more public interface functions
	
	Boolean doDialog					// data dialogs
	(
		int modeless = 0				// optionally, post a modeless box
	);
	TDialog *getDialog ();
	void undoDialog
	(
		TDialog *pd
	);
	void updateFields ();
	Boolean mappedDialog ();			// special data dialogs
	int write
	(
		FILE *fptr
	);
	int read
	(
		FILE *fptr
	);

protected:								// internal functions

	TView *makeField
	(
		DataType datatype,				// exact type of boxy data item
		char **strings,					// points list of button-strings
		TPoint dul,						// upper-left corner of box
		TPoint dlr,						// lower-right corner of box
		int maxlength,					// for TInputLine(s)
		Range& code,					// internal range of units
		char *format,					// format of display
		int mapped,						// is data mapping to be employed?
		Range& user,					// user-viewable range of units
		MouseMap& map					// mouse- and arrow-mappings
	);
	TView *makeButtons
	(
		DataType type,					// type of (we hope) boxy data
		char **list,					// points list of button-strings
		TPoint dul,						// upper-left corner of box
		TPoint dlr						// lower-right corner of box
	);
	void buttonsOKCancel
	(
		TDialog *pd,
		TPoint lr
	);

private:								// logical functions for types

	Boolean isBox
	(
		DataType type
	);
	Boolean isNumeric
	(
		DataType type
	);
	Boolean isString
	(
		DataType type
	);
	Boolean isControl
	(
		DataType type
	);
	Boolean isExtended
	(
		DataType type
	);

private:

	DeskTop *boxTop;					// indirect access to deskTop

private:

	int useDataMap;						// quick flag
	int useFieldMap;					// quick flag
	ExtendedDescriptor *dataMap;		// access to special types
	MappedFieldDescriptor *fieldMap;	// access to special types

	int dataSize;						// size of structure

	MainBox *dialogWindow;				// nested structure (tv_menu.h)
	MappedBox *mappedWindow;			// nested structure (tv_menu.h)
	GeneralDialogData dialogData;		// points general dialog structure

	TDialog *modelessDialog;			// last modeless dialog

};

#endif													// } DLG_BOX_h

