/************************************************************************
**
** @(#)lightbit.h		06/27/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
** C++
**
** Edit with tab-stops = 4.
**
**		Defines the ScreenCombo enumeration.
**
*************************************************************************/

#if !defined(LIGHTBIT_h)								// { LIGHTBIT_h
#define LIGHTBIT_h


/************************************************************************
** LightState
**
**		This enumeration lists a set of bit-masks.  They summarize what
** should be done with the light.  Some lights will have functions
** devoted to their use exclusively.  Some lights will simply use a
** text function, while others will use a graphics function.
**
**		Examine the LIGHT_IS_SPECIAL and LIGHT_IS_TEXT bits, and study
** the following table, which is a possible truth table for those 
** bits:
**
**	SPECIAL		TEXT		Example Function
**
**	   0		 0			Draw a box using the color, size, and location
**							determined by looking them up in an array
**							of box-specifying structures.
**
**	   0		 1			Draw the text using the color, size, font,
**							and location found in the look-up table.
**
**	   1		 0			Call a function that determines the light
**							number in a switch() statement or hash
**							table, and calls its function.  This is
**							useful for certain oddball lights.
**
**	   1		 1			Can handle this the same way.
**
**		The offsets into a lookup table can be defined however the
** derived class wants:
**
**		a.	Use the index of the light itself.  This is suitable for
**			a small number of lights, all having the same general
**			characteristics.
**
**		b.	Use the index from the LIGHT_PARAMETERS bits.  This is useful
**			in two cases:
**
**			  i. We have a lot of different kinds of lights, but only
**				 a few of each kind.  We don't want to make a parameter
**				 array of full size for each kind.
**
**			 ii. Many of the lights are text strings.
**
**		The ResponseBoxInterior class function drawThis() partially
** handles the cases listed above.  For the SPECIAL bit being off,
** it obtains the table-lookup offset ("lpboffset") from the
** lights bitMaskList[] entry, and calls drawGraphic(lpboffset) or
** drawText(lpboffset).
**
**		If the special bit is on, it calls drawSpecial(light_number).
** Of course, all these can be overridden.
**
*************************************************************************/

typedef enum
{
	LIGHT_IS_SPECIAL	= 0x0001,	// light has special function
	LIGHT_IS_TEXT		= 0x0002,	// light drawn with text, not graphics
	LIGHT_IS_BACKGROUND	= 0x0004,	// reserved for derived class's use
	LIGHT_IS_RSRV0008	= 0x0008,	// reserved for derived class's use

	LIGHT_IS_ON			= 0x0010,	// light is currently on
	LIGHT_IS_TOGGLE		= 0x0020,	// light is toggle-able or useable
	LIGHT_IS_RSRV0040	= 0x0040,	// reserved for derived class's use
	LIGHT_IS_RSRV0080	= 0x0080,	// reserved for derived class's use

	LIGHT_PARAMETERS	= 0xFF00,	// light may have extra parameters

} LightState;


#endif													// } LIGHTBIT_h
