/************************************************************************
**
** @(#)points.h		06/23/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
** C++ only
**
** Edit with tab-stops = 4.
**
**		Defines the Point and Rectangle classes.  All the definitions
** herein are very similar to the TPoint and TRectangle classes defined
** by Turbo Vision.  They are redefined here for those modules that
** don't use Turbo Vision stuff, but are somewhat compatible.
**
*************************************************************************/

#if !defined(POINTS_h)									// { POINTS_h
#define POINTS_h


inline int
imin
(
	int a,
	int b
)
{
    return (a>b) ? b : a ;
}

inline int
imax
(
	int a,
	int b
)
{
    return (a<b) ? b : a ;
}


enum boolean
{
	false,
	true
};



class Point
{

public:

    Point & operator +=
	(
		const Point &addend
	);
	Point & operator -=
	(
		const Point &subtrahend
	);
    friend Point operator -
	(
		const Point &one,
		const Point &two
	);
    friend Point operator +
	(
		const Point &one,
		const Point &two
	);
    friend int operator ==
	(
		const Point &one,
		const Point &two
	);
    friend int operator !=
	(
		const Point &one,
		const Point &two
	);

    int x, y;

};


inline Point &
Point::operator +=
(
	const Point &addend
)
{
	x += addend.x;
	y += addend.y;
	return *this;
}


inline Point &
Point::operator -=
(
	const Point &subtrahend
)
{
    x -= subtrahend.x;
    y -= subtrahend.y;
    return *this;
}


#if defined(USE_STREAMS)								// { USE_STREAMS

inline ipstream & operator >>
(
	ipstream &is,
	Point &p
)
{
	return is >> p.x >> p.y;
}

inline ipstream & operator >>
(
	ipstream &is,
	Point *&p
)
{
	return is >> p->x >> p->y;
}

inline opstream & operator <<
(
	opstream &os,
	Point &p
)
{
	return os << p.x << p.y;
}

inline opstream & operator <<
(
	opstream &os,
	Point *p
)
{
	return os << p->x << p->y;
}

#endif												// } USE_STREAMS


class Rectangle
{

public:

    Rectangle
	(
		int ax,
		int ay,
		int bx,
		int by
	);
    Rectangle
	(
		Point p1,
		Point p2
	);
    Rectangle ();

    void move
	(
		int aDX,
		int aDY
	);
    void grow
	(
		int aDX,
		int aDY
	);
    void intersect
	(
		const Rectangle &r
	);
    void Union
	(
		const Rectangle &r
	);
    boolean contains
	(
		const Point &p
	) const;
    boolean operator ==
	(
		const Rectangle &r
	) const;
    boolean operator !=
	(
		const Rectangle &r
	) const;
    boolean isEmpty ();

    Point a, b;

};

inline Rectangle::Rectangle
(
	int ax,
	int ay,
	int bx,
	int by
)
{
    a.x = ax;
    a.y = ay;
    b.x = bx;
    b.y = by;
}

inline Rectangle::Rectangle
(
	Point p1,
	Point p2
)
{
    a = p1;
    b = p2;
}

inline Rectangle::Rectangle()
{
}

inline void Rectangle::move
(
	int aDX,
	int aDY
)
{
    a.x += aDX;
    a.y += aDY;
    b.x += aDX;
    b.y += aDY;
}

inline void Rectangle::grow
(
	int aDX,
	int aDY
)
{
    a.x -= aDX;
    a.y -= aDY;
    b.x += aDX;
    b.y += aDY;
}

inline void Rectangle::intersect
(
	const Rectangle &r
)
{
    a.x = imax(a.x, r.a.x);
    a.y = imax(a.y, r.a.y);
    b.x = imin(b.x, r.b.x);
    b.y = imin(b.y, r.b.y);
}

inline void Rectangle::Union
(
	const Rectangle &r
)
{
    a.x = imin(a.x, r.a.x);
    a.y = imin(a.y, r.a.y);
    b.x = imax(b.x, r.b.x);
    b.y = imax(b.y, r.b.y);
}

inline boolean Rectangle::contains
(
	const Point &p
) const
{
    return boolean(p.x >= a.x && p.x < b.x && p.y >= a.y && p.y < b.y);
}

inline boolean Rectangle::operator ==
(
	const Rectangle &r
) const
{
    return boolean(a == r.a && b == r.b);
}

inline boolean Rectangle::operator !=
(
	const Rectangle &r
) const
{
    return boolean(!(*this == r));
}

inline boolean Rectangle::isEmpty ()
{
    return boolean(a.x >= b.x || a.y >= b.y);
}

#if defined(USE_STREAMS)								// { USE_STREAMS

inline ipstream & operator >>
(
	ipstream &is,
	Rectangle &r
)
{
	return is >> r.a >> r.b;
}

inline ipstream & operator >>
(
	ipstream &is,
	Rectangle *&r
)
{
	return is >> r->a >> r->b;
}

inline opstream & operator <<
(
	opstream &os,
	Rectangle &r
)
{
	return os << r.a << r.b;
}

inline opstream & operator <<
(
	opstream &os,
	Rectangle *r
)
{
	return os << r->a << r->b;
}

#endif												// } USE_STREAMS

#endif													// } POINTS_h
