/************************************************************************
**
** @(#)rbox_btk.cpp		01/07/96		Chris Ahlstrom
**
** C++
**
** Edit with tab-stops set to 4.
**
**		This module defines the ResponseBGITokenInterior class, which
** objectifies the abstract class ResponseBoxInterior for use with
** the Borland BGI library.
**
**		The procedure displays a layout like the following:
**
**		 -----------------------------------------
**		|  B  |  C  |  D  |  F  |  G  |  H  |  J  |
**		 -----------------------------------------
**		|  M  |  N  |  P  |  Qu |  S  |  T  |  Th |
**		 -----------------------------------------
**		|  W  |  X  |  Y  |  Z  |  Dz |
**		 -----------------------------
**
**			READY									SELECT
**
**								LISTEN
**
**	The screen is internally divided into a 7 x 5 grid.  The box
** numbers are thus layed out as follows on the screen:
**
**		 1	 2	 3	 4	 5	 6	 7
**		 8	 9	10	11	12	13	14
**		15	16	17	18	19	20	21
**		22	23	24	25	26	27	28
**		29	30	31	32	33	34	35
**
** When box numbers are provided, some box numbers might not be
** used; no token will appear in that slot.
**
**	cleardevice();						// Borland routine
**
**		Note that monoMouseCursor() and mainMouseCursor() do nothing;
** tmtomono.asm must not be correct, I guess.  Sigh.
**
*************************************************************************/

//////////////////////////////////
//	#define USE_DEBUG_CODE	1	//
//////////////////////////////////

#if defined(USE_DEBUG_CODE)						// { USE_DEBUG_CODE

#include <stdio.h>			// sprintf() debugging

#define BLANKER								\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"		\
	"\xDB" "\xDB" "\xDB" "\xDB" "\xDB"

#endif											// } USE_DEBUG_CODE


#define RBOX_BTK_cpp

#include <dos.h>			// delay()
#include <math.h>			// floor()
#include <graphics.h>		// Borland graphics functions

#include "far.h"			// define FAR
#include "rbox_btk.h"		// ResponseBGITokenInterior class and ancestors

extern "C"
{
	#include "tmouse.h"		// a few mouse routines [mouseShowCursor()]
}


/************************************************************************
** Colors
*************************************************************************/

#define LINE_COLOR				WHITE		// color of grid lines
#define INTERVAL_COLOR			WHITE		// color of "Listen" light
#define READY_COLOR				RED			// color of "Ready" light
#define ANSWER_COLOR			GREEN		// color of "Answer" light
#define BACKGROUND_COLOR		BROWN		// color of background

#define MONO_LINE_COLOR			WHITE		// any color except black will do
#define MONO_INTERVAL_COLOR		MONO_LINE_COLOR
#define MONO_READY_COLOR		MONO_LINE_COLOR
#define MONO_ANSWER_COLOR		MONO_LINE_COLOR
#define MONO_BACKGROUND_COLOR	BLACK


/************************************************************************
** TokenLightCode
**
**		An internal enumeration.
**
*************************************************************************/

typedef enum
{
	TLC_BANNERLIGHT,
	TLC_READYLIGHT,
	TLC_ANSWERLIGHT,
	TLC_FAILURELIGHT

} TokenLightCode;



/************************************************************************
**
** Wygonski values (as opposed to values for BGI on the Hercules):
**
**		#define XSPACE		100 	// x-axis spacing
**		#define YSPACE		 50		// y-axis spacing
**		#define XLOFS		  2		// x-line offset from left
**		#define YLOFS		 20		// y-line offset from top
**		#define XPOFS		 25		// x-phoneme offset in box
**		#define YPOFS		 35		// y-phoneme offset in box
**
*************************************************************************/

#define XSPACE		100 	// x-axis spacing
#define YSPACE		 50		// y-axis spacing
#define XLOFS		  2		// x-line offset from left
#define YLOFS		 30		// y-line offset from top
#define XPOFS		 30		// x-phoneme offset in box
#define YPOFS		 14		// y-phoneme offset in box

#define TB_COLUMNS	  7		// maximum columns of boxes in grid
#define TB_ROWS		  5		// maximum rows of boxes in grid


/************************************************************************
** hercBitMap[] and vgaBitMap[]
**
**		This represents the monochrome cursor, which this module will
** take care of.  It was created by using the HGCPIXEL utility of the
** FACE_TV library as follows:
**
**				hgcpixel -s mono h.bit		(saucer shape)
**				hgcpixel -c mono h.bit		(crosshair shape)
**				hgcpixel -c vga v.bit		(crosshair shape for VGA)
**
** Then, the declaration was copied from h.bit and placed here.  Note
** that we cannot implement the cursor in the subInterior object
** ResponseBGITokenInterior [at least, not without more parameter-
** passing], since the screen cursor must follow the mouse cursor.
**
**		Our calculated size of image is 32 width x 16 height = 512
** pixels (64 bytes).
**
*************************************************************************/

#if defined(USE_SAUCER_SHAPE)

/************************************************************************
** Monochromo saucer shape
*************************************************************************/

const int hercBitWidth	= 32;
const int hercBitHeight	= 16;
static const unsigned char FAR hercBitMap[70] =
{
	0x1F, 0x00, 0x0F, 0x00, 	/* two words (width-1 and height-1)		*/

	0x00, 0x01, 0x80, 0x00, 
	0x00, 0x07, 0xC0, 0x00, 
	0x00, 0x19, 0x30, 0x00, 
	0x00, 0x61, 0x0C, 0x00, 
	0x01, 0x81, 0x03, 0x00, 
	0x06, 0x01, 0x00, 0xC0, 
	0x18, 0x01, 0x00, 0x30, 
	0x60, 0x01, 0x00, 0x0C, 
	0xFF, 0xFF, 0xFF, 0xFF, 
	0x30, 0x01, 0x00, 0x0C, 
	0x0C, 0x01, 0x00, 0x30, 
	0x03, 0x01, 0x01, 0xC0, 
	0x00, 0xE1, 0x06, 0x00, 
	0x00, 0x19, 0x18, 0x00, 
	0x00, 0x07, 0x60, 0x00, 
	0x00, 0x01, 0x80, 0x00, 

	0x2C, 0x20					/* not sure why these are here!			*/
};

#endif


/************************************************************************
** Monochromo crosshair shape
**
**		Our calculated size of image is 32 width x 16 height = 512
** pixels (64 bytes).
**
*************************************************************************/

const int hercBitWidth	= 32;
const int hercBitHeight	= 16;
const unsigned char FAR hercBitMap[70] =
{
	0x1F, 0x00, 0x0F, 0x00, 	/* two words (width-1 and height-1)	*/

	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x07, 0xC0, 0x00, 
	0x00, 0x09, 0x20, 0x00, 
	0x00, 0x11, 0x10, 0x00, 
	0xFF, 0xFF, 0xFF, 0xFF, 
	0x00, 0x11, 0x10, 0x00, 
	0x00, 0x09, 0x20, 0x00, 
	0x00, 0x07, 0xC0, 0x00, 
	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x01, 0x00, 0x00, 
	0x00, 0x01, 0x00, 0x00, 

	0x30, 0x30 
};


/************************************************************************
** VGA crosshair shape
**
**		Our calculated size of image is 28 width x 9 height = 252 pixels
** (31 bytes x 4 = 124).
**
*************************************************************************/

const int vgaBitWidth	= 28;
const int vgaBitHeight	=  9;
const unsigned char FAR vgaBitMap[134] =
{
	0x1B, 0x00, 0x07, 0x00, 	/* two words (width-1 and height-1)	*/

	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 
	0x00, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 
	0xFF, 0xFF, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 0xF0, 
	0xFF, 0xFF, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 0xF0, 
	0x00, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 
	0x00, 0x1F, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 
	0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 

	0x2C, 0x20
};



/************************************************************************
** ResponseBGITokenInterior constructor and destructor.
*************************************************************************/

ResponseBGITokenInterior::ResponseBGITokenInterior
(
	TokenLayout *tokens,
	int tokencount,
	const char * /* atitle */ ,					// optional title
	ScreenCombo combo
) :
	ResponseBoxInteriorBase (),
	Tokens					(tokens),
	tokenCount				(tokencount),
	monoChrome				(0),
	lineColor				(LINE_COLOR),
	intervalColor			(INTERVAL_COLOR),
	readyColor				(READY_COLOR),
	answerColor				(ANSWER_COLOR),
	backgroundColor			(BACKGROUND_COLOR),
	bgiControl				(new BGIManager(combo)),
	xAxisSpacing			(XSPACE),
	yAxisSpacing			(YSPACE),
	xLineOffset				(XLOFS),
	yLineOffset				(YLOFS),
	xPhonemeOffset			(XPOFS),
	yPhonemeOffset			(YPOFS),
	handleOwnCursor			(1),				// 0 does not work yet
	monoCoordinates			(0),				// 1 does not work yet
	cursorImage				(vgaBitMap),
	bitWidth				(vgaBitWidth),
	bitHeight				(vgaBitHeight),
	minMouseX				(0),
	minMouseY				(0),
	maxMouseX				(0),
	maxMouseY				(0),
	oldMouseX				(0),
	oldMouseY				(0),
	imageOn					(0)
{
	lightCount = tokencount;

	if (bgiControl && bgiControl->graphicsOK())
	{
		ScreenCombo temp = bgiControl->monitorType();
		if ((temp == BGI_USE_HERC) || (temp == BGI_USE_MONO_PATCH))
			monoChrome = 1;

		if (monoChrome)
		{
			lineColor		= MONO_LINE_COLOR;
			intervalColor	= MONO_INTERVAL_COLOR;
			readyColor		= MONO_READY_COLOR;
			answerColor		= MONO_ANSWER_COLOR;
			backgroundColor	= MONO_BACKGROUND_COLOR;
			cursorImage		= hercBitMap;
			bitWidth		= hercBitWidth;
			bitHeight		= hercBitHeight;
		}
		else
			yPhonemeOffset -= 17;			// correction in placement

		bitWidth   /= 2;					// offset for drawing cursor
		bitHeight  /= 2;					// offset for drawing cursor
		minMouseX	= bitWidth;
		minMouseY	= bitHeight;
		maxMouseX	= 640 - 1 - bitWidth;	// BREAKABLE CODE
		maxMouseY	= 200 - 1 - bitHeight;	// BREAKABLE CODE

		setbkcolor(backgroundColor);				// set color
		setcolor(intervalColor);					// default foreground
		settextstyle(TRIPLEX_FONT, HORIZ_DIR, 3);	// hard-wired
		startState();								// do usual stuff

		if (monoCoordinates)
		{
			if (monoChrome)
				monoMouseCursor(0);			// fix page 0 to use mono-cursor
			mouseReset();
		}
		if (!handleOwnCursor)
			mouseShowCursor();
	}
}


/************************************************************************
** ResponseBGITokenInterior destructor
**
**		I'm wondering if it's safe to put the responseOff(1) call
** inside the ResponseInterior destructor instead.  I don't remember
** the order of destructor calls in inheritance.
**
*************************************************************************/

ResponseBGITokenInterior::~ResponseBGITokenInterior ()
{
	if (monoCoordinates)
	{
		if (monoChrome)
			mainMouseCursor();				// fix to use old cursor
		mouseReset();
	}
	if (!handleOwnCursor)
		mouseHideCursor();

	responseOff(1);							// turn off all lights
	if (bgiControl)
		delete bgiControl;
}


/************************************************************************
** allOff()
*************************************************************************/

void
ResponseBGITokenInterior::allOff ()
{
	responseOff(1);					// turn off all lights
}


/************************************************************************
** startState()
**
**		This function sets up all the lights, normally making them
** all visible, making it clear to the user that things are ready to
** go.  If some hardware problem occurs, a non-zero error code is
** returned.
**
**		Note that we can't call this function in the constructor
** because lightCount and bitMaskList won't have good values yet.
**
*************************************************************************/

int
ResponseBGITokenInterior::startState ()
{
	int errcode = 0;

	if (bgiControl)
	{
		noMoreBanner = 0;							// allow banner (again)

		/****************************************************************
		** Set up the subject screen
		*****************************************************************/

		int startx	= 9999;
		int starty	= 9999;
		int endx	= 0;
		int endy	= 0;

		for (int ti = 0; ti < tokenCount; ti++)
		{
			int v;

			fillTokenBox(Tokens[ti].phoneme, Tokens[ti].box);

			v = (Tokens[ti].box-1) % TB_COLUMNS;

			if (v < startx)
				startx = v;

			if ((v+1) > endx)
				endx = v + 1;

			v = Tokens[ti].box / TB_COLUMNS;

			if (v < starty)
				starty = v;

			if ((v+1) > endy)
				endy = v + 1;

			/************************************************************
			** startx	= min(startx,(Tokens[i].box-1)%TB_COLUMNS);
			** endx	= max(endx,((Tokens[i].box-1)%TB_COLUMNS)+1);
			** starty	= min(starty,Tokens[i].box/TB_COLUMNS);
			** endy	= max(endy,Tokens[i].box/TB_COLUMNS+1);
			*************************************************************/
		}
		drawGrid(startx, endx, starty, endy);

		errcode = readyLight(LIGHT_ON);				// turn it on
		errcode = answerLight(LIGHT_ON);			// turn it on
		errcode = bannerLight(LIGHT_ON);			// ca 12/08/94
		errcode = drawThis();						// draw them

		readyForUse = 1;	// next time drawThis() can call drawing code
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** responseOff()
**
**		Turns off all lights, excluding the banner and frame lights,
** and, internally, the background light.
**
**		If desired, can have them all turned off by passing a
** non-zero allofthem parameter.
**
*************************************************************************/

int
ResponseBGITokenInterior::responseOff
(
	int // allofthem					// not necessary or able!
)
{
	int errcode = 0;

	if (bgiControl)
	{
		cleardevice();					// Borland routine
		errcode = drawThis();
	}
	else
		errcode = 1;

	return errcode;
}


/************************************************************************
** readyLight()
**
**		Turns on the Ready light.
**
**		The "onstate" parameter is handled a little differently than in
** the ResponseBGIInterior class (for example) version of readyLight(),
** where it is called "offem" instead.  The value TURN_THEM_OFF
** does nothing; the rest of the lights stay on.  The value TURN_THEM_OFF
** is already 1 (TRICKY CODE), so it is simply passed on.
**
*************************************************************************/

int
ResponseBGITokenInterior::readyLight
(
	int onstate
)
{
	noMoreBanner = 1;			// turn off main banner now

	return drawSpecial(TLC_READYLIGHT, onstate);
}


/************************************************************************
** failureLight()
**
**		See the discussion for readyLight().
**
*************************************************************************/

int
ResponseBGITokenInterior::failureLight
(
	int onstate
)
{
	noMoreBanner = 0;			// turn banner on (emphasize error locus)

	return drawSpecial(TLC_FAILURELIGHT, onstate);
}


/************************************************************************
** bannerLight()
*************************************************************************/

int
ResponseBGITokenInterior::bannerLight
(
	int onstate
)
{
	return drawSpecial(TLC_BANNERLIGHT, onstate);
}


/************************************************************************
** answerLight()
**
**		See the discussion for readyLight().
**
*************************************************************************/

int
ResponseBGITokenInterior::answerLight
(
	int onstate
)
{
	return drawSpecial(TLC_ANSWERLIGHT, onstate);
}


/************************************************************************
** intervalLight()
**
**		For speech tokens, we currently support no interval lights.
** However, if the caller provides an explicit parameter of onstate = 0,
** then the appropriate token will flash once.
**
**		In any case, the readyLight() and answerLight() should be
** turned off.
**
*************************************************************************/

int
ResponseBGITokenInterior::intervalLight
(
	int toknumber,			// from 0 to tokenCount-1
	int onstate
)
{
	int errcode = 0;

	errcode = readyLight(LIGHT_OFF);				// turn it off
	errcode = answerLight(LIGHT_OFF);				// turn it off

	if (onstate == LIGHT_ON)						// 0 was provided
	{
		errcode = drawText(toknumber, LIGHT_OFF);	// turn off the box
		delay(300);									// wait a bit
		errcode = drawText(toknumber, LIGHT_ON);	// turn it back on
	}
	return errcode;
}


/************************************************************************
** responseLight()
**
**		Briefly turns off the response-token.
**
*************************************************************************/

int
ResponseBGITokenInterior::responseLight
(
	int toknumber,			// from 0 to tokenCount-1
	int // onstate
)
{
	int errcode = 0;

	drawText(toknumber, LIGHT_OFF);
	delay(300);
	drawText(toknumber, LIGHT_ON);

	errcode = drawThis();
	return errcode;
}


/************************************************************************
** wrongLight()
**
**		Turns off all lights except the BOX_WRONG_RESPONSE.
**
*************************************************************************/

int
ResponseBGITokenInterior::wrongLight
(
	int // onstate
)
{
	return drawThis();
}


/************************************************************************
** rightLight()
**
**		Turns off all lights except the BOX_RIGHT_RESPONSE.
**
*************************************************************************/

int
ResponseBGITokenInterior::rightLight
(
	int // onstate
)
{
	return drawThis();
}


/************************************************************************
** feedbackLight()
*************************************************************************/

int
ResponseBGITokenInterior::feedbackLight
(
	int toknumber,
	int // response
)
{
	return responseLight(toknumber);
}


/************************************************************************
** drawGraphic()
**
**
** drawText()
**
**		This routine doesn't draw all the text.  It just draws the
** text in each cell in the grid.
**
**
** drawSpecial()
**
*************************************************************************/

int
ResponseBGITokenInterior::drawGraphic
(
	unsigned /* lpboffset */,	// grid component to draw
	unsigned /* onstate */		// turning it on or off (or maybe even reverse?)
) 
{
	return 0;
}


int
ResponseBGITokenInterior::drawText
(
	unsigned toknumber,			// index [re 0] of text to draw
	unsigned onstate
) 
{
	settextstyle(TRIPLEX_FONT, HORIZ_DIR, 3);	// hard-wired

	if (toknumber < tokenCount)
	{
		if (onstate)
			fillTokenBox(Tokens[toknumber].phoneme, Tokens[toknumber].box);
		else
			clearTokenBox(Tokens[toknumber].phoneme, Tokens[toknumber].box);
	}
	else
		fillTokenBox("???", (TB_COLUMNS*TB_ROWS+1));

	return 0;
}


int
ResponseBGITokenInterior::drawSpecial
(
	int lightnumber,
	unsigned onstate
)
{
	int errcode = 0;
	char *outstring = "Light Error";
	int offsetx = 0;
	int offsety = 0;
	int color;

	switch (lightnumber)
	{
	case TLC_BANNERLIGHT:

		outstring	= "Press & hold Left button to stop; press again to start";
		offsetx		=   0;
		offsety		=   0;
		color		= intervalColor;
		break;

	case TLC_ANSWERLIGHT:

		outstring	= "Select";
		offsetx		= 520;
		offsety		= 290;					// was 270;
		color		= answerColor;
		break;

	case TLC_FAILURELIGHT:

		outstring	= "General failure; please quit";
		offsetx		=  20;
		offsety		= 300;
		color		= intervalColor;
		break;

	case TLC_READYLIGHT:

		outstring	= "Ready";
		offsetx		= 120;
		offsety		= 290;					// was 270;
		color		= readyColor;
		break;
	}

	if (onstate == LIGHT_OFF)				// do we want to clear the text?
		setcolor(backgroundColor);			// yes, use Borland's way
	else
		setcolor(color);					// no, use proper color

	outtextxy
	(
		bgiControl->screenX(offsetx), bgiControl->screenY(offsety), outstring
	);

	setcolor(intervalColor);

	errcode = drawThis();

	return errcode;
}



/************************************************************************
** drawThis()
**
**		Since we don't keep a record of the state of the interior,
** this function does nothing.
**
**		It could serve as a central place for handling problems.
**
*************************************************************************/

int
ResponseBGITokenInterior::drawThis ()
{
	return 0;
}


/************************************************************************
** drawBoxCorrect() and drawBoxIncorrect() might go away
*************************************************************************/


void
ResponseBGITokenInterior::drawBoxCorrect ()
{
	int x[2*4];			// (x[n], x[n+1]) ::= (x[n], y[n])

	x[0] = 300; x[1] = 300;
	x[2] = 315; x[3] = 315;
	x[4] = 310; x[5] = 320;
	x[6] = 295; x[7] = 305;
	fillpoly(4, x);

	x[0] = 310; x[1] = 310;
	x[2] = 330; x[3] = 290;
	x[4] = 335; x[5] = 295;
	x[6] = 315; x[7] = 315;
	fillpoly(4, x);
}


void
ResponseBGITokenInterior::drawBoxIncorrect ()
{
	int x[2*4];			// (x[n], x[n+1]) ::= (x[n], y[n])

	x[0] = 295; x[1] = 295;
	x[2] = 320; x[3] = 320;
	x[4] = 315; x[5] = 325;
	x[6] = 290; x[7] = 300;
	fillpoly(4, x);

	x[0] = 315; x[1] = 295;
	x[2] = 320; x[3] = 300;
	x[4] = 295; x[5] = 325;
	x[6] = 290; x[7] = 320;
	fillpoly(4, x);
}


void
ResponseBGITokenInterior::drawBoxClick ()
{
	outtextxy(20, 20, "Click a mouse button to begin");
}


/***********************************************************************
** drawGrid()
**
**		Draws the background grid.  It is a pretty hard-wired routine
** at present.
**
************************************************************************/

void
ResponseBGITokenInterior::drawGrid
(
	int sx,
	int ex,
	int sy,
	int ey
)
{
	int x1, x2, y1, y2;							/* line coordinates */
	int i;

	x1 = xLineOffset + (sx * xAxisSpacing);
	y1 = yLineOffset + (sy * yAxisSpacing);
	x2 = xLineOffset + (ex * xAxisSpacing);
	for (i = sy; i <= ey; i++)					/* draw horiz lines */
	{
		line
		(
			bgiControl->screenX(x1), bgiControl->screenY(y1),
			bgiControl->screenX(x2), bgiControl->screenY(y1)
		);
		y1 += yAxisSpacing;
	} 
	x1 = xLineOffset + (sx * xAxisSpacing);
	y1 = yLineOffset + (sy * yAxisSpacing);
	x2 = x1;
	y2 = yLineOffset + (ey * yAxisSpacing);
	for (i = sx; i <= ex; i++)					/* draw vert lines */
	{
		line
		(
			bgiControl->screenX(x1), bgiControl->screenY(y1),
			bgiControl->screenX(x2), bgiControl->screenY(y2)
		);
		x1 += xAxisSpacing;
		x2 += xAxisSpacing;
	} 
}


/***********************************************************************
** fillTokenBox()
************************************************************************/

void
ResponseBGITokenInterior::fillTokenBox
(
	char *ostr,
	int box
)
{
	int x, y;

	box--;

	x = (box % TB_COLUMNS) * xAxisSpacing;
	y = (box / TB_COLUMNS) * yAxisSpacing;

	x += xPhonemeOffset + xLineOffset;
	y += yPhonemeOffset + yLineOffset; 

	outtextxy(bgiControl->screenX(x), bgiControl->screenY(y), ostr);
}


/***********************************************************************
** clearTokenBox()
************************************************************************/

void
ResponseBGITokenInterior::clearTokenBox
(
	char *ostr,
	int box
)
{
	setcolor(backgroundColor);
	fillTokenBox(ostr, box);
	setcolor(intervalColor);
}


/***********************************************************************
** cursorOn()
** cursorOff()
** cursorMove()
**
**		cursorOn() sets up the initial cursor parameters and mouse
** position, or it turns the image on.
**
**		If good fractional values (greater than or equal to 0) are
** given, the cursor is moved to that location, but is /not/ displayed.
** Otherwise, the cursor is simply turned on whereever it happens to be.
**
**		cursorOff() removes the image, if it is present.
**
**		The normal sequence of events is to call cursorOn(a, b), where
** (a,b) is the initial coordinate for the center of the cursor.  This
** sets up the cursor, but does not display it.  Later, the caller
** can then call cursorOn() with no parameters, to display it.
**
**		cursorMove() then redisplays the cursor in a new location.  The
** caller can do this a number of times, then call cursorOff() to turn
** off the cursor.
**
************************************************************************/

void
ResponseBGITokenInterior::cursorOn
(
	double xfraction,		// ranges from 0 to 1, relative start of cursor
	double yfraction		// ranges from 0 to 1, relative start of cursor
)
{
	if (xfraction >= 0.0)	// if 0, don't change cursor/mouse positions
	{
		int x = int((maxMouseX - minMouseX - 1.0) * xfraction) + minMouseX;
		int y = int((maxMouseY - minMouseX - 1.0) * yfraction) + minMouseY;

		setMousePixel(x, y);					// set pixel position

		x = bgiControl->mouseX(x) - bitWidth;	// fix re screen and bit image
		y = bgiControl->mouseY(y) - bitHeight;	// fix re screen and bit image

		oldMouseX = x;
		oldMouseY = y;
	}
	else if (handleOwnCursor)
	{
		putimage							// plot first image
		(
			oldMouseX,		// left
			oldMouseY,		// top,
			cursorImage,	// bitmap,
			XOR_PUT			// bit opcode
		);
		imageOn = 1;						// it can be erased
	}
}


void
ResponseBGITokenInterior::cursorOff ()
{
	if (imageOn)								// only erase if present!
	{
		putimage								// erase old image
		(
			oldMouseX, oldMouseY, cursorImage, XOR_PUT
		);
		imageOn = 0;							// now it ain't there, dude.
	}
}


void
ResponseBGITokenInterior::cursorMove
(
	int x,
	int y
)
{
	if (x < minMouseX)						// clip horizontally
		x = minMouseX;
	else if (x > maxMouseX)
		x = maxMouseX;

	if (y < minMouseY)						// clip vertically
		y = minMouseY;
	else if (y > maxMouseY)
		y = maxMouseY;

#if defined(USE_DEBUG_CODE)							// { USE_DEBUG_CODE

	char lineout[48];

	(void) sprintf(lineout, BLANKER);
	setcolor(backgroundColor);
	outtextxy(0, 0, lineout);
	setcolor(intervalColor);
	(void) sprintf(lineout, "VGA(x,y)=(%3d,%3d)        ", x, y);
	outtextxy(0, 0, lineout);

#endif												// } USE_DEBUG_CODE

	x = bgiControl->mouseX(x) - bitWidth;		// fix re screen and bit image
	y = bgiControl->mouseY(y) - bitHeight;		// fix re screen and bit image

	if (handleOwnCursor && ((x != oldMouseX) || (y != oldMouseY)))
	{
		if (imageOn)
		{
			putimage							// erase old image
			(
				oldMouseX, oldMouseY, cursorImage, XOR_PUT
			);
		}
		putimage							// plot new image
		(
			x, y, cursorImage, XOR_PUT
		);
		imageOn = 1;
		oldMouseX = x;
		oldMouseY = y;
	}
}


/***********************************************************************
** getBoxNumber()
**
**		Accepts a pair of mouse coordinates, and converts them to
** a box number ranging from 1 to TB_COLUMNS*TB_ROWS [35].
**
**		Because we have no control over the mouse driver, it will always
** use the coordinates of the VGA screen.  Therefore, we must always
** convert them to Hercules units.  [All our grid/token coordinates
** are in Hercules units, as inherited from Wygonski.]
**
************************************************************************/

int
ResponseBGITokenInterior::getBoxNumber
(
	int x,		// x from MOUSE code (usually from 0 to 639)
	int y		// y from MOUSE code (usually from 0 to 199)
)
{
	int sx, sy;							// scaled X & Y
	int boxnum;

	sx = (int) floor((double) bgiControl->hercX(x) + 0.5);
	sy = (int) floor((double) bgiControl->hercY(y) + 0.5);

	if
	(
		(sx < xLineOffset) || (sx >= (xLineOffset+TB_COLUMNS*xAxisSpacing)) ||
		(sy < yLineOffset) || (sy >= (yLineOffset+TB_ROWS*yAxisSpacing))
	)
	{
		boxnum = 0;									// indicates out of box
	}
	else
	{
		int xgrid, ygrid;				// x & y grid coords

		xgrid = (sx - xLineOffset) / xAxisSpacing;	// integer divide wanted
		ygrid = (sy - yLineOffset) / yAxisSpacing;	// integer divide wanted
		boxnum = xgrid + (TB_COLUMNS * ygrid) + 1;
	} 

#if defined(USE_DEBUG_CODE)							// { USE_DEBUG_CODE

	char lineout[48];

	(void) sprintf(lineout, BLANKER);
	setcolor(backgroundColor);
	outtextxy(0, 160, lineout);
	setcolor(intervalColor);
	(void) sprintf(lineout, "Box #=%3d    ", boxnum);
	outtextxy(0, 160, lineout);

#endif												// } USE_DEBUG_CODE

	return boxnum;
}

