/************************************************************************
**
** @(#)rbox_dev.h		10/17/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
**	C++
**
**	Edit this module with tab-stops of 4.
**
**		This module supports generic response-box devices.  It should
** supercede the ResponseDevice class, which was very restrictive.
** The modules to supercede are:
**
**		RBUTTONS
**		RESPWIND
**
**		The model for a ResponseBoxDevice is pretty simple.  We have
** manipulator functions that call functions for the response-button
** and response-light-drawing classes.
**
*************************************************************************/

#if !defined(RBOX_DEV_h)								// { RBOX_DEV_h
#define RBOX_DEV_h


#include <dos.h>
#include <conio.h>

#if !defined(__TTYPES_H)
#include <ttypes.h>		// "Boolean" Turbo Vision definition
#endif

#define Uses_TDeskTop
#define Uses_TPalette
#define Uses_TRect
#define Uses_TView
#define Uses_TWindow
#include <tv.h>			// Turbo Vision code galore!!!

#include "rbuttons.h"		// ResponseButton class

#define TV_PALETTES			// get access to _PAL colors
#include "rbox_int.h"		// ResponseBoxInterior class


/************************************************************************
** ResponseBoxDevice
**
**		We override the version of startScreen, in
** in order to provide the extra function of waiting for the user.
**
**		We add a responseFeedback() routine to wait for a key press
** and show the feedback lights.
**
**		Note the defining of Turbo Vision color palettes, too, the
** same way as in the old module, respwind.h, except we put it
** in the main module.
**
*************************************************************************/

class ResponseBoxDevice : public TWindow
{

public:

	ResponseBoxDevice
	(
		ResponseType device,
		const TRect& bounds,
		const char *aTitle,
		short aNumber
	);
	~ResponseBoxDevice();

	ResponseBoxSet startScreen				// bring up initial screen
	(
		int proceed = 0						// if 1, don't need to wait
	);
	void readyLight();						// Warning! Trial is starting!
	void responseOff();
	void intervalLight
	(
		ushort interval
	);
	void answerLight();
	int responseFeedback
	(
		int stimulus
	);
	void pause ();							// waits for a key response
	int isKeyed();							// true if a key is pressed
	int keyAbort();							// true if last key-->aborted

	ResponseBoxInterior *subInterior;		// access to interior

	virtual TPalette &getPalette() const;

protected:

	ResponseButton *buttons;				// handles subject's input

};


#endif													// } RBOX_DEV_h
