/************************************************************************
**
** @(#)rbox_dtk.h		11/28/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
**	C++; edit this module with tab-stops of 4.
**
**		This module supports generic response-box devices.  It should
** supercede the ResponseDevice class, which was very restrictive.
** The modules to supercede are:
**
**		RBUTTONS
**		RESPWIND
**
**		The model for a ResponseBGITokenDevice is pretty simple.  We have
** manipulator functions that call functions for the response-button
** and response-light-drawing classes.
**
*************************************************************************/

#if !defined(RBOX_DTK_h)								// { RBOX_DTK_h
#define RBOX_DTK_h


#include "rbuttons.h"		// ResponseButton class
#include "rbox_bas.h"		// ResponseBoxDeviceBase class
#include "rbox_btk.h"		// ResponseBGITokenInterior class


/************************************************************************
** TOKEN_KEY_OFFSET
**
**		The normal response is from 1 (BUTTON_1 only) to 7 (ALL_BUTTONS).
** But, for token classes, it is from 1 to a large number, such as 64.
** Therefore, we depend on the caller understanding the situation -- we
** keep the original set of responses, so that BOTH_BUTTONS==3 has its
** usual meaning.  The other responses are offset upward by 10, and the
** caller must correct for this fact.  TRICKY CODE.
**
*************************************************************************/

#define TOKEN_KEY_OFFSET	10	// offset the selection for "encoding"


/************************************************************************
** ResponseBGITokenDevice
**
**		We override the version of startScreen, in
** in order to provide the extra function of waiting for the user.
**
**		We add a responseFeedback() routine to wait for a key press
** and show the feedback lights.
**
*************************************************************************/

class ResponseBGITokenDevice : public ResponseBoxDeviceBase
{

public:

	ResponseBGITokenDevice
	(
		TokenLayout *tokens,
		int tokencount,
		ResponseType device,
		const Rectangle &bounds,
		const char *atitle,			
		ScreenCombo combo,
		int selectionfeedback = 0,		// don't show true selection...
		int tactilefeedback = 1			// ...just show subject's choice
	);

	virtual ~ResponseBGITokenDevice();

	virtual ResponseBoxSet startScreen
	(
		int proceed = 0
	);
	virtual void intervalLight
	(
		unsigned interval
	);
	virtual int responseFeedback
	(
		int stimulus
	);

protected:

	TokenLayout *tokenList;				// token strings and box-numbers
	int tokenCount;						// number of token in the list
	int selectionFeedback;				// set to blink the true token
	int tactileFeedback;				// set to blink user-selected token
	int oldX;							// old coordinates, saves time
	int oldY;

};


#endif													// } RBOX_DTK_h
