/************************************************************************
**
** @(#)rbox_inb.cpp		11/17/94		Chris Ahlstrom
**
** C++ version, requires Turbo Vision by Borland
**
**		This module defines the ResponseBoxInteriorBase class, which
** defines a standard set of calls (interface) for other classes.
**
*************************************************************************/

#define RBOX_INB_cpp

#include <conio.h>			// cputs()

#include "rbox_inb.h"		// ResponseBoxInteriorBase class


/************************************************************************
** ResponseBoxInteriorBase constructor and destructor.
**
**		Note that, since we allocate a semi-permanent textBox, it
** might prove wise to add an assignment operator and copy constructor.
**
*************************************************************************/

ResponseBoxInteriorBase::ResponseBoxInteriorBase ()
 :
	lightCount		(0),				// max number lights in class
	readyForUse		(0),				// flag raised later
	graphicsReady	(0),				// flag must be raised later
	noMoreBanner	(0)					// flag raised later
{
	//////////////////////////////////////////////////////////////////////
	// Do not call startState() here!  Call it in the derived class!    //
	//////////////////////////////////////////////////////////////////////
}


ResponseBoxInteriorBase::~ResponseBoxInteriorBase ()
{
}


/************************************************************************
** getBoxNumber()
**
**		To allow for interaction with the mouse to be /visible/ in
** derived classes, we provide these cursor-drawing functions, which
** should convert the mouse coordinates into a visible picture.
**
**		NOTE:  It would probably be better at some time to move many of
** the member values related to mouse and cursor from the rbox_btk
** module into this module, so that all ResponseBoxInteriorBase classes
** can take advantage of them.  LATER.
**
*************************************************************************/

void
ResponseBoxInteriorBase::cursorOn
(
	double /* xfraction */,		// 0 to 1, relative start of cursor
	double /* yfraction */		// 0 to 1, relative start of cursor
)
{
}


void
ResponseBoxInteriorBase::cursorOff ()
{
}


void
ResponseBoxInteriorBase::cursorMove	
(
	int /* x */,
	int /* y */
)
{
}


/************************************************************************
** getBoxNumber()
**
**		To allow for interaction with the mouse in derived classes,
** we provide this function, which should convert the mouse coordinates
** provided into some kind of integer number.
**
*************************************************************************/

int
ResponseBoxInteriorBase::getBoxNumber
(
	int /* x */ ,
	int /* y */
)
{
	return 0;
}


/************************************************************************
** allOff()
**
**		A handy-dandy utility.  Turns off all the necessary lights.
** The name is a bit misleading, as responseOff() turns off /everything/,
** while allOff() just turns off the non-background lights.
**
** startState()
**
**		This function sets up all the lights, normally making them
** all visible, making it clear to the user that things are ready to
** go.  If some hardware problem occurs, a non-zero error code is
** returned.
**
**		Note that we can't call this function in the constructor
** because the derived class has to do its own setups first.
**
** drawGraphic()
** drawText()
** drawSpecial()
**
**		These functions draw one light.  They must be defined in the
** derived class, even if the function does nothing.  Each function
** is passed a number to tell it which item to draw, and how it is to
** be drawn.
**
**
** -------------------
** Patterns of lights
** -------------------
**
**		The following functions all should set up the proper lights for
** the function, then call drawThis().  The meanings of these functions
** are class-independent, though the nature of the lights in each
** class can differ.
**
**		Each routine must call drawThis() when finished.
**
**
** responseOff()
**
**		Turns off all lights, excluding the banner and frame lights,
** and, internally, the background light.
**
**		If desired, can have them all turned off by passing a
** non-zero allofthem parameter.
**
**
** readyLight()
**
**		Turns on the READY_LIGHT.
**
**		If the "offem" parameter is non-zero (the default value is 1),
** then all the lights except the BOX_READY_LIGHT, BOX_BACKGROUND_LIGHT, and
** BOX_FRAME_LIGHT are turned off.
**
**
** failureLight()
**
**		Turns on the BOX_FAILURE_LIGHT.
**
**		If the "offem" parameter is non-zero (the default value is 1),
** then all the lights except the BOX_FAILURE_LIGHT, BOX_BACKGROUND_LIGHT, and
** BOX_FRAME_LIGHT are turned off.
**
**
** bannerLight()
**
**		Turns on the BOX_BANNER_LIGHT.
**
**		If the "offem" parameter is non-zero (the default value is 1),
** then all the lights except the BOX_BANNER_LIGHT, BOX_BACKGROUND_LIGHT, and
** BOX_FRAME_LIGHT are turned off.
**
**
** answerLight()
**
**		Turns off all lights except the BOX_ANSWER_LIGHT.
**
**
** intervalLight()
**
**		Turns off all lights except one of these lights:  BOX_INTERVAL_1,
** BOX_INTERVAL_2, or BOX_INTERVAL_3.  The parameter ranges from 1 to 3
** for 3AFC tasks, or more for speech token tasks.
**
**
** responseLight()
**
**		Turns off all lights except one of these lights:  BOX_RESPONSE_1,
** BOX_RESPONSE_2, or BOX_RESPONSE_3.  The parameter ranges from 1 to 3.
**
**
** wrongLight()
**
**
**		Turns off all lights except the BOX_WRONG_RESPONSE.
**
** rightLight()
**
**
**		Turns off all lights except the BOX_RIGHT_RESPONSE.
**
** feedbackLight()
**
**
**		Turns off all lights except one of the interval lights and
** one of the response lights.
**
**
** drawThis()
**
**		Draws the current state of the screen.  This means redrawing
** all the lights in the interior.  To save time, it reviews the old
** state of the screen, to see what needs changing.
**
*************************************************************************/

