/************************************************************************
**
** @(#)rbox_int.h		11/14/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
** C++; edit with tab-stops = 4.
**
**		Defines the ResponseBoxInterior class, which supercedes the
** ResponseInterior class of the RESPINTR modules.  This class is
** a base class, independent of video support.
**
**		Since it is derived from an abstract base class, all of the
** routines are overridden here.  In addition, we define many routines
** to support the concept of a list of bit-masks to summarize how
** various lights are to be drawn.  (Such a list is defined in
** rbox2afc.hdt, for example.)
**
**		The derived classes define what is drawn, and how it is drawn.
** See rbox_int.cpp for a discussion of how things can be drawn.
**
*************************************************************************/

#if !defined(RBOX_INT_h)								// { RBOX_INT_h
#define RBOX_INT_h


#include "rbox_inb.h"		// ResponseBoxInteriorBase abstract base class
#include "rbox_afc.h"		// ResponseBoxLightIndex and other types
#include "lightbit.h"		// LightState


/************************************************************************
** Light lists
**
**		We provide support for 64 lights.  This can be increased to
** 256 with no changes in coding.
**
**		To store the last states of all the lights, we compress the
** on-bit (LIGHT_IS_ON) for each light into an array of unsigned
** chars (8 bits).
**
*************************************************************************/

#define LIGHT_COUNT_MAX		64					// use 256 if need be
#define OLD_COUNT		(LIGHT_COUNT_MAX / 8)	// for on-bit storage


/************************************************************************
** ResponseBoxInterior
**
**		Since a ResponseBox is allocated in the constructor, we really
** need to add an assignment operator and copy constructor. Later...
** (famous last word).
**
*************************************************************************/

class ResponseBoxInterior : public ResponseBoxInteriorBase
{

public:

	ResponseBoxInterior();					// constructor

	virtual ~ResponseBoxInterior ();		// destructor

	virtual int drawThis ();				// redraws whole interior
	virtual int startState ();				// turn on all the lights
	virtual int intervalLight
	(
		int interval,
		int offem = TURN_THEM_OFF
	);
	virtual int responseLight
	(
		int response,
		int offem = TURN_THEM_OFF
	);

public:										// unchanging public functions

	virtual int responseOff					// turn off all but a couple
	(
		int allofthem = 0					// turn those off, too
	);
	virtual int lightOn
	(
		ResponseBoxLightIndex lite,
		int offem = TURN_THEM_OFF
	);
	virtual int readyLight
	(
		int offem = TURN_THEM_OFF
	);
	virtual int failureLight
	(
		int offem = TURN_THEM_OFF
	);
	virtual int bannerLight
	(
		int offem = TURN_THEM_OFF
	);
	virtual int answerLight
	(
		int offem = TURN_THEM_OFF
	);
	virtual int wrongLight
	(
		int offem = TURN_THEM_OFF
	);
	virtual int rightLight
	(
		int offem = TURN_THEM_OFF
	);
	virtual int feedbackLight
	(
		int interval,
		int response
	);

public:										// pure virtual functions

	virtual int drawSpecial					// redraw special light
	(
		int lightnumber,
		unsigned onstate
	) = 0;
	virtual int drawText
	(
		unsigned lpboffset,
		unsigned onstate
	) = 0;
	virtual int drawGraphic
	(
		unsigned lpboffset,
		unsigned onstate
	) = 0;

protected:									// internal support

	virtual void allOff ();					// utility function

protected:									// inline functions

	unsigned getState						// access to oldMaskList[]
	(
		int entity							// ranges from 0 to 256
	);
	void setState							// access to oldMaskList[]
	(
		int entity,							// ranges from 0 to 256
		int value							// 0 or non-zero only
	);

protected:

	unsigned *bitMaskList;					// derived class's bitMaskList
	int lightCount;							// max number lights supported
	int readyForUse;						// can't draw until ready
	int graphicsReady;						// can't draw if no graphics
	int noMoreBanner;						// only want banner at start

	unsigned char oldMaskList[OLD_COUNT];	// condensed list of on-bits
	static const int lightCountMax;			// max number lights ever supported

};


#endif													// } RBOX_INT_h
