/************************************************************************
**
** @(#)rbox_tv.h		06/22/94		Chris Ahlstrom
**
** ---------------------------
**  73340.26@compuserve.com
** ---------------------------
**
** C++
**
**	Edit with tab-stops = 4.
**
**		This module supports showing annunciator lights in a TV window.
** The model is very strict, applicable only to 2- or 3-alternative
** forced choice tasks in psychoacoustics.  See the CPP file.
**
**		The FACE_TV (and FACE_WIN) versions are a little different from
** the code extracted from the MODAPP 2AFC-experiment code.
**
*************************************************************************/

#if !defined(RBOX_TV_h)									// { RBOX_TV_h
#define RBOX_TV_h

#if !defined(__TTYPES_H)
#include <ttypes.h>			// "Boolean" Turbo Vision definition
#endif

#define Uses_TRect
#define Uses_TView
#include <tv.h>				// Turbo Vision code galore!!!


#define TV_PALETTES			// this module uses TV windows
#include "rbox_afc.h"		// ResponseBoxLight structures for TV/VGA/Hercules


/************************************************************************
** ResponseBoxTView
**
**		Since a ResponseBox is allocated in the constructor, we really
** need to add an assignment operator and copy constructor. Later...
** (famous last word).
**
*************************************************************************/

class ResponseBoxTView : public TView
{

public:

	ResponseBoxTView					// constructor
	(
		ResponseBoxLight resplightparameters[],
		int resplightcount,
		const TRect &bounds				// size of the window
	);

	~ResponseBoxTView ();				// destructor

	virtual void draw ();				// override TView::draw()

	void boxAspectConvert				// convert all interior lights
	(
		TRect *wbox						// window box to hold it all
	);
	void lightAspectConvert				// converts a light to text units
	(
		TRect *wbox,					// window box into which light goes
		ResponseBoxLight& light,		// pixel-coordinates of light
		ResponseBoxLight& text			// text-coordinate-offsets of light
	);
	void textAspectConvert				// converts text to text units
	(
		TRect *wbox,					// window box into which light goes
		ResponseBoxText& light,			// pixel-coordinates of light
		ResponseBoxText& text			// text-coordinate-offsets of light
	);
	void lightDraw(int lnumber);		// draws one light
	void light							// sets drawing status of each light
	(
		unsigned char lightmask			// only a light with its bit on appears
	);
	void lightOn(int lnumber);			// sets a light's status to BRESP_ON
	void lightOff(int lnumber);			// sets a light's status to BRESP_OFF

protected:

	int respRows;
	int respColumns;
	int respPixelX;
	int respPixelY;

	ResponseBoxLight *graphBox;			// light-layout in "virtual" units
	ResponseBoxLight *textBox;				// light-layout in text units

	int intervalsUsed;					// either 2 or 3 (3 NOT READY)
	int lightCount;						// number of lights really handled
	int lightStatus[BRESP_LIGHTS_MAX];	// holds current light status

};


#endif													// } RBOX_TV_h
