/************************************************************************
**
** @(#)rbuttons.h		11/17/94		Chris Ahlstrom
**
**		This module interfaces C++ with code for handling various kinds
** of button devices.
**
**		Also defines constants for the Turbo Vision implementation of
** response support.
**
*************************************************************************/

#if !defined(RBUTTONS_h)								// { RBUTTONS_h
#define RBUTTONS_h


/************************************************************************
** ResponseType
**
**		Simply selects the device to use.  We don't allow simultaneous
** interchangeable use of all devices.
**
**		ALT_MOUSE_RESPONSE and ALT_MOUSE_POINT_RESPONSE are only
** available on systems that have two mouse ports.  It tries to spawn
** and run the "C:\MOUSE\MOUSE" command to swap mice to be used while
** the response device is active.
**
**		At present, this spawning does not work properly (not sure why!).
** So, instead, we provide a batch file MBOOTH.BAT which must be
** run before running the application program [MLAB.BAT switches it
** back].  But then the 2nd mouse is always the mouse used; the primary
** mouse cannot be used again until MLAB.BAT is run.
**
*************************************************************************/

typedef enum
{
	MOUSE_RESPONSE				= 0,	// left or right buttons
	KEYBOARD_RESPONSE,					// 1, 2, or E key
	KOALA_RESPONSE,						// left or right buttons
	MOUSE_POINT_RESPONSE,				// click in 1st, 2nd, or 3rd section
	ALT_MOUSE_RESPONSE,					// left or right buttons, 2nd port
	ALT_MOUSE_POINT_RESPONSE			// click in regions, 2nd port

} ResponseType;


/************************************************************************
** Response
**
**		All responses are mapped onto a two-button response device.
**
**		BUTTON_3 and ALL_BUTTONS are not yet supported.
**
** KeyResponse
**
**		The keys as mapped onto Response values.
**
*************************************************************************/

typedef enum
{
	NO_BUTTON			= 0x00,
	BUTTON_1			= 0x01,
	BUTTON_2			= 0x02,
	BUTTON_3			= 0x04,
	BOTH_BUTTONS		= (BUTTON_1+BUTTON_2),
	ALL_BUTTONS			= (BOTH_BUTTONS+BUTTON_3)

} Response;

typedef enum
{
	KEY_NO_BUTTON		= 0,
	KEY_BUTTON_1		= '1',
	KEY_BUTTON_2		= '2',
	KEY_BUTTON_3		= '3',
	KEY_BOTH_BUTTONS	= 'B',
	KEY_BOTH_ESCAPE		= '\x1B',
	KEY_ALL_BUTTONS		= 'A'

} ResponseKey;


/************************************************************************
** ResponseButton base class
**
**		The protected member responseDevice is of little use at the
** moment.
**
**		The protected member lastResponse is useful perhaps for
** doing tricky but too-tricky things (e.g. two-key double clicks),
** or for saving the status so that a Windows mouse-handler routine
** can decide what to do with it.
**
*************************************************************************/

class ResponseButton
{

public:

	ResponseButton							// constructor
	(
		ResponseType device					// kind of device
	);

	virtual ~ResponseButton ();				// destructor

	int checkAbort ();
	int getLastResponse ()
	{
		return (int) lastResponse;
	}

	virtual int readKey ();					// default support is keyboard
	virtual int waitKey ();
	virtual int readCoordinates
	(
		int &xvalue,
		int &yvalue
	);

protected:

	ResponseType responseDevice;			// default device is keyboard
	Response lastResponse;					// previous response

};


#endif													// } RBUTTONS_h

